/*! edge-common v0.1.1 by Sebastian Software <s.werner@sebastian-software.de> */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

require('lean-nodent-runtime');
var _set = _interopDefault(require('lodash/set'));
var path = require('path');
var cosmiconfig = _interopDefault(require('cosmiconfig'));
var appRootDir = require('app-root-dir');
var toBool = _interopDefault(require('yn'));
var jsome = _interopDefault(require('jsome'));
var chalk = _interopDefault(require('chalk'));
var notifier = _interopDefault(require('node-notifier'));

// Export common understanding of what ROOT is
var ROOT = appRootDir.get();

var appPkg = require(ROOT + "/package.json");

var NAME = appPkg.name;
var VERSION = appPkg.version;
var LOGPREFIX = NAME + "-" + VERSION + ":";

var SCHEMA = {
  verbose: {
    type: "boolean",
    "default": false
  },

  quiet: {
    type: "boolean",
    "default": false
  },

  entry: {
    serverMain: {
      type: "path",
      "default": "src/server/index.js"
    },
    clientMain: {
      type: "path",
      "default": "src/client/index.js"
    },
    serverVendor: {
      type: "path",
      "default": "src/server/vendor.js"
    },
    clientVendor: {
      type: "path",
      "default": "src/server/vendor.js"
    }
  },

  output: {
    server: {
      type: "path",
      "default": "build/server"
    },
    client: {
      type: "path",
      "default": "build/client"
    },
    "public": {
      type: "url",
      "default": "/static/"
    }
  },

  build: {
    enableSourceMaps: {
      type: "boolean",
      "default": true
    },
    bundleCompression: {
      type: "string",
      "default": "uglify"
    },
    useCacheLoader: {
      type: "boolean",
      "default": true
    },
    babelEnvPrefix: {
      type: "string",
      "default": "edge"
    }
  },

  locale: {
    "default": {
      type: "string",
      "default": "en-US"
    },
    supported: {
      type: "array",
      "default": ["en-US", "es-ES", "de-DE"]
    }
  },

  hook: {
    webpack: {
      type: "script",
      "default": "hooks/webpack.js"
    }
  }
};

function loadConfig() {
  var $args = arguments;return new Promise(function ($return, $error) {
    var prefix, flags, configLoader, configResult, config, configRoot;
    prefix = $args.length > 0 && $args[0] !== undefined ? $args[0] : "edge";
    flags = $args.length > 1 && $args[1] !== undefined ? $args[1] : {};

    configLoader = cosmiconfig(prefix, {
      // allow extensions on rc files
      rcExtensions: true,

      // Force stop at project root folder
      stopDir: ROOT
    });

    var $Try_1_Post = function () {

      config = configResult.config;

      for (var key in flags) {
        _set(config, key, flags[key]);
      }

      return processConfig(config, SCHEMA).then(function ($await_9) {

        configRoot = path.relative(ROOT, configResult.filepath);

        if (config.verbose) {
          console.log("Configuration from " + configRoot + ":");
          jsome(config);
        }

        return $return({
          config: config,
          root: configRoot
        });
      }.$asyncbind(this, $error), $error);
    }.$asyncbind(this, $error);var $Try_1_Catch = function (parsingError) {
      throw new Error("Error parsing config file: " + parsingError + ". Root: " + ROOT + ".");
    }.$asyncbind(this, $error);

    try {
      return configLoader.load(ROOT).then(function ($await_10) {
        configResult = $await_10;
        return $Try_1_Post();
      }.$asyncbind(this, $Try_1_Catch), $Try_1_Catch);
    } catch (parsingError) {
      $Try_1_Catch(parsingError);
    }
  }.$asyncbind(this));
}

function processConfig(config, schema) {
  return new Promise(function ($return, $error) {
    var specs, value;
    var $idx_3,
        $in_4 = [];for ($idx_3 in schema) $in_4.push($idx_3);var key;return Function.$asyncbind.trampoline(this, $Loop_5_exit, $Loop_5, $error, true)($Loop_5);

    function $Loop_5() {
      if ($in_4.length) {
        key = $in_4.shift();
        specs = schema[key];
        value = config[key] || {};

        if (specs.type == null) {
          return processConfig(value, specs).then(function ($await_11) {
            config[key] = $await_11 || value;
            return $If_7.call(this);
          }.$asyncbind(this, $error), $error);
        } else {
          return processEntry(value, specs).then(function ($await_12) {
            config[key] = $await_12;
            return $If_7.call(this);
          }.$asyncbind(this, $error), $error);
        }

        function $If_7() {
          return $Loop_5;
        }
      } else return [1];
    }

    function $Loop_5_exit() {
      return $return();
    }
  }.$asyncbind(this));
}

var identityFnt = function (item) {
  return item;
};

/* eslint-disable complexity */
function processEntry(value, specs) {
  return new Promise(function ($return, $error) {
    var parsed;

    parsed = void 0;

    switch (specs.type) {
      case "string":
        if (typeof value !== "string") {
          return $error(new Error("Invalid config value for type string!"));
        }
        return $return(String(value));

      case "number":
        parsed = parseFloat(value, 10);
        if (isNaN(parsed)) {
          return $error(new Error("Invalid config value for type number!"));
        }
        return $return(parsed);

      case "boolean":
        return $return(toBool(value));

      case "path":
        if (typeof value !== "string") {
          return $error(new Error("Invalid config value for type path (string required)!"));
        }
        return $return(path.resolve(ROOT, value));

      case "url":
        if (typeof value !== "string") {
          return $error(new Error("Invalid config value for type url (string required)!"));
        }
        return $return(value);

      case "array":
        if (!(value instanceof Array)) {
          return $error(new Error("Invalid config value for type array!"));
        }
        return $return(value);

      case "script":
        var $Try_2_Post = function () {

          return $return(parsed ? parsed["default"] || parsed : identityFnt);
        }.$asyncbind(this, $error);var $Try_2_Catch = function (err) {
          parsed = null;
          return $Try_2_Post();
        }.$asyncbind(this, $error);
        try {
          return new Promise(function (resolve) {
            require.ensure([], function (require) {
              resolve(require(resolve(ROOT, value)));
            });
          }).then(function ($await_13) {
            parsed = $await_13;
            return $Try_2_Post();
          }.$asyncbind(this, $Try_2_Catch), $Try_2_Catch);
        } catch (err) {
          $Try_2_Catch(err);
        }default:
        return $error(new Error("Unsupported entry type in config " + specs.type + "!"));
    }
    return $return();
  }.$asyncbind(this));
}

function colorize(message) {
  var level = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

  switch (level) {
    case "warn":
      return chalk.yellow(message);

    case "error":
      return chalk.red(message);

    case "info":
      return chalk.green(message);

    default:
      return message;
  }
}

function notify(message) {
  var level = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

  notifier.notify({
    title: NAME + "-" + VERSION,
    message: message
  });

  /* eslint-disable no-console */
  var consoleMessage = chalk.bold(NAME) + ": " + colorize(message, level);
  switch (level) {
    case "warn":
      console.warn(consoleMessage);
      break;

    case "error":
      console.error(consoleMessage);
      break;

    case "info":
      console.log(consoleMessage);
      break;

    default:
      console.log(consoleMessage);
  }
}

exports.NAME = NAME;
exports.VERSION = VERSION;
exports.LOGPREFIX = LOGPREFIX;
exports.SCHEMA = SCHEMA;
exports.loadConfig = loadConfig;
exports.notify = notify;
exports.colorize = colorize;
//# sourceMappingURL=node.cjs.js.map
